<?php

namespace Drupal\entity_404\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatchInterface;

/**
 * Deny access if an en entity doesn't have a full view.
 */
class HasFullView implements AccessInterface {

  /**
   * The access result reason.
   */
  public const ACCESS_RESULT_REASON = 'Entity has no full view';

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected EntityDisplayRepositoryInterface $entityDisplayRepository;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   */
  public function __construct(EntityDisplayRepositoryInterface $entity_display_repository) {
    $this->entityDisplayRepository = $entity_display_repository;
  }

  /**
   * Check if the entity a full view.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(RouteMatchInterface $route_match): AccessResultInterface {
    // Try to get the entity.
    $parameters = $route_match->getParameters()->all();
    $entity = reset($parameters);

    if (!$entity instanceof ContentEntityInterface) {
      return AccessResult::allowed();
    }

    // Ensure the full view exists.
    $view_modes = $this->entityDisplayRepository->getViewModeOptionsByBundle(
      $entity->getEntityTypeId(),
      $entity->bundle()
    );

    if (isset($view_modes['full'])) {
      $result = AccessResult::allowed();
    }
    else {
      $result = AccessResult::forbidden(self::ACCESS_RESULT_REASON);
    }

    return $result->addCacheTags([
      "config:core.entity_view_display.{$entity->getEntityTypeId()}.{$entity->bundle()}.full",
    ]);
  }

}
